package com.nsgk.ruralWeb.location;

import android.annotation.SuppressLint;
import android.app.Activity;
import android.content.Context;
import android.location.Criteria;
import android.location.Location;
import android.location.LocationManager;
import android.os.Build;
import android.os.CancellationSignal;
import android.util.Log;
import android.widget.Toast;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.TimeUnit;

public class NSSystemLocation
{
    private static final String ID_TAG = NSSystemLocation.class.getName();

    private final Context        m_context;
    private       NSLocationInfo m_lastLocation = null;
    private volatile CancellationSignal m_cancellationSignal;
    private volatile CompletableFuture<Location> m_future;

    public NSSystemLocation(Context context)
    {
        m_context = context;
    }

    @SuppressLint("MissingPermission")
    public NSLocationInfo Read(String provider, int timeout)
    {
        if(Build.VERSION.SDK_INT < Build.VERSION_CODES.R)
        {
            Log.w(ID_TAG, "获取当前定位请求Android 11+");
            return null;
        }

        NSLocationInfo loc = null;
        try
        {
            LocationManager lm = (LocationManager) m_context.getSystemService(Context.LOCATION_SERVICE);
            if(null != provider)
            {
                if(LocationManager.GPS_PROVIDER.equals(provider))
                {
                    if(!lm.isProviderEnabled(provider))
                    {
                        Log.w(ID_TAG, "GPS定位提供器未启用, 尝试网络定位器");
                        provider = LocationManager.NETWORK_PROVIDER;
                        if(!lm.isProviderEnabled(provider))
                        {
                            Log.w(ID_TAG, "网络定位提供器也未启用");
                            provider = null;
                        }
                    }
                }
                else if(LocationManager.NETWORK_PROVIDER.equals(provider))
                {
                    Log.w(ID_TAG, "网络定位提供器未启用, 尝试GPS定位器");
                    if(!lm.isProviderEnabled(provider))
                    {
                        provider = LocationManager.GPS_PROVIDER;
                        if(!lm.isProviderEnabled(provider))
                        {
                            Log.w(ID_TAG, "GPS定位提供器也未启用");
                            provider = null;
                        }
                    }
                }
            }

            if(null == provider)
            {
                Criteria criteria = new Criteria();
                criteria.setAccuracy(Criteria.ACCURACY_FINE); // 高精度
//            criteria.setPowerRequirement(Criteria.POWER_HIGH);
//            criteria.setAltitudeRequired(false);
                provider = lm.getBestProvider(criteria, true);
            }
            Log.i(ID_TAG, "使用高精度定位提供器: " + provider);

            if(null == provider)
            {
                Log.i(ID_TAG, "无法获取高精度定位提供器");
                return null;
            }

            m_cancellationSignal = new CancellationSignal();
            m_future = new CompletableFuture<>();
            lm.getCurrentLocation(provider, m_cancellationSignal, m_context.getMainExecutor(), m_future::complete);
            Location location;
            if(timeout > 0)
                location = m_future.get(timeout, TimeUnit.MILLISECONDS);
            else
                location = m_future.get();
            m_cancellationSignal = null;
            m_future = null;
            if(null == location)
            {
                Log.i(ID_TAG, "无法使用高精度提供器定位");
                return null;
            }
            loc = new NSLocationInfo(provider, location.getLongitude(), location.getLatitude());
            Log.i(ID_TAG, "使用高精度提供器获取定位: " + loc);
            m_lastLocation = loc;
        }
        catch(Throwable e)
        {
            e.printStackTrace();
        }
        finally
        {
            m_cancellationSignal = null;
            m_future = null;
        }
        return loc;
    }

    private void ShowToast(final String message, int duration)
    {
        ((Activity)m_context).runOnUiThread((new Runnable() {
            @Override
            public void run()
            {
                Toast.makeText(m_context, message, duration).show();
            }
        }));
    }

    public void TerminateRead()
    {
        CancellationSignal signal = m_cancellationSignal;
        if(null != signal && !signal.isCanceled())
        {
            Log.i(ID_TAG, "系统中止定位: 1. 取消定位");
            signal.cancel();
        }
        CompletableFuture<Location> future = m_future;
        if(null != future && !future.isCancelled())
        {
            Log.i(ID_TAG, "系统中止定位: 2. 取消等待结果");
            future.cancel(true);
        }
    }

    public void Shutdown()
    {
        Log.i(ID_TAG, "销毁系统定位");
        TerminateRead();
    }

    public NSLocationInfo GetLastLocation()
    {
        return m_lastLocation;
    }
}